'use strict';

exports.__esModule = true;

var _react = require('react');

var React = _interopRequireWildcard(_react);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * `<Completion />` elements describe different data sources. Multiple can be
 * used in the same [`<AutoComplete />`][AutoComplete] component.
 */
var Completion = function (_React$Component) {
  _inherits(Completion, _React$Component);

  function Completion() {
    _classCallCheck(this, Completion);

    return _possibleConstructorReturn(this, _React$Component.apply(this, arguments));
  }

  Completion.prototype.render = function render() {
    return null;
  };

  return Completion;
}(React.Component);

Completion.propTypes = {
  /**
   * String that triggers this completion type.
   */
  trigger: React.PropTypes.oneOfType([React.PropTypes.string, React.PropTypes.instanceOf(RegExp)]).isRequired,
  /**
   * Minimum amount of characters typed before suggestions will be given.
   */
  minLength: React.PropTypes.number,
  /**
   * Regex to extract the current completion value from the input. Can also be
   * used to "validate" the current completion value, so no suggestions will
   * be provided if it's "invalid".
   *
   * Uses the first capture group as the value to be completed, or the full
   * match if there are no capture groups. For example:
   *  - /.*(@.*?)$/ + "Hello @ReA" → "@ReA"
   *  - /\w+$/ + "This is sp" → "sp"
   */
  regex: React.PropTypes.instanceOf(RegExp),
  /**
   * Function that renders a single suggestion.
   *
   * @param {Object} suggestion - Suggestion descriptor.
   * @param {string} suggestion.key - Unique key for the suggestion element.
   *     See [Dynamic Children](https://facebook.github.io/react/docs/multiple-components.html#dynamic-children)
   *     for details.
   * @param {*} suggestion.value - Completion value of this suggestion.
   * @param {boolean} suggestion.selected - Whether this suggestion is
   *     currently selected.
   * @param {function} suggestion.select - Autocomplete this suggestion.
   * @returns {element}
   */
  renderSuggestion: React.PropTypes.func,
  /**
   * Get an array of possible completions.
   *
   * @default Searches the `completions` prop.
   *
   * @param {string} matchingValue - Current value to be completed, as
   *     extracted using `props.regex`.
   * @param {Object} props - Props of this `<Completion />` element.
   * @returns {Array.<*>}
   */
  getCompletions: React.PropTypes.func,
  /**
   * Optional array of completion values. This can be used if all possible
   * completions are known beforehand. If provided, a default `getCompletions`
   * function that searches this array will be used.
   */
  completions: React.PropTypes.array,
  /**
   * Transform a completion value to a string that will be inserted into the
   * input component. By default, uses `${props.trigger}${value} `. (Note the
   * space at the end! If you want to add a space once a completion is
   * inserted, add it here.)
   *
   * @param {*} value - Completion value.
   * @param {Object} props - Props of this `<Completion />` element.
   * @returns {string}
   */
  getText: React.PropTypes.func
};
Completion.defaultProps = {
  minLength: 3,
  getCompletions: function getCompletions(value, _ref) {
    var trigger = _ref.trigger;
    var completions = _ref.completions;

    var compare = value.substr(trigger.length).toLowerCase();
    return completions.filter(function (completion) {
      return completion.substr(0, compare.length).toLowerCase() === compare;
    });
  },
  getText: function getText(value, _ref2) {
    var trigger = _ref2.trigger;
    return '' + trigger + value + ' ';
  }
};
exports.default = Completion;