'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var escapeStringRegExp = _interopDefault(require('escape-string-regexp'));

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

/**
 * `<Completion />` elements describe different data sources. Multiple can be
 * used in the same [`<AutoComplete />`][AutoComplete] component.
 */

var Completion = function (_React$Component) {
  inherits(Completion, _React$Component);

  function Completion() {
    classCallCheck(this, Completion);
    return possibleConstructorReturn(this, _React$Component.apply(this, arguments));
  }

  Completion.prototype.render = function render() {
    return null;
  };

  return Completion;
}(React.Component);

Completion.defaultProps = {
  minLength: 3,
  getCompletions: function getCompletions(value, _ref) {
    var trigger = _ref.trigger;
    var completions = _ref.completions;

    var compare = value.substr(trigger.length).toLowerCase();
    return completions.filter(function (completion) {
      return completion.substr(0, compare.length).toLowerCase() === compare;
    });
  },
  getText: function getText(value, _ref2) {
    var trigger = _ref2.trigger;
    return '' + trigger + value + ' ';
  }
};
process.env.NODE_ENV !== "production" ? Completion.propTypes = {
  /**
   * String that triggers this completion type.
   */
  trigger: React.PropTypes.oneOfType([React.PropTypes.string, React.PropTypes.instanceOf(RegExp)]).isRequired,
  /**
   * Minimum amount of characters typed before suggestions will be given.
   */
  minLength: React.PropTypes.number,
  /**
   * Regex to extract the current completion value from the input. Can also be
   * used to "validate" the current completion value, so no suggestions will
   * be provided if it's "invalid".
   *
   * Uses the first capture group as the value to be completed, or the full
   * match if there are no capture groups. For example:
   *  - /.*(@.*?)$/ + "Hello @ReA" → "@ReA"
   *  - /\w+$/ + "This is sp" → "sp"
   */
  regex: React.PropTypes.instanceOf(RegExp),
  /**
   * Function that renders a single suggestion.
   *
   * @param {Object} suggestion - Suggestion descriptor.
   * @param {string} suggestion.key - Unique key for the suggestion element.
   *     See [Dynamic Children](https://facebook.github.io/react/docs/multiple-components.html#dynamic-children)
   *     for details.
   * @param {*} suggestion.value - Completion value of this suggestion.
   * @param {boolean} suggestion.selected - Whether this suggestion is
   *     currently selected.
   * @param {function} suggestion.select - Autocomplete this suggestion.
   * @returns {element}
   */
  renderSuggestion: React.PropTypes.func,
  /**
   * Get an array of possible completions.
   *
   * @default Searches the `completions` prop.
   *
   * @param {string} matchingValue - Current value to be completed, as
   *     extracted using `props.regex`.
   * @param {Object} props - Props of this `<Completion />` element.
   * @returns {Array.<*>}
   */
  getCompletions: React.PropTypes.func,
  /**
   * Optional array of completion values. This can be used if all possible
   * completions are known beforehand. If provided, a default `getCompletions`
   * function that searches this array will be used.
   */
  completions: React.PropTypes.array,
  /**
   * Transform a completion value to a string that will be inserted into the
   * input component. By default, uses `${props.trigger}${value} `. (Note the
   * space at the end! If you want to add a space once a completion is
   * inserted, add it here.)
   *
   * @param {*} value - Completion value.
   * @param {Object} props - Props of this `<Completion />` element.
   * @returns {string}
   */
  getText: React.PropTypes.func
} : void 0;

function getRegex(_ref) {
  var regex = _ref.regex;
  var trigger = _ref.trigger;

  if (regex) {
    return regex;
  }
  var t = escapeStringRegExp(trigger);
  return new RegExp('.*(' + t + '.*?)$');
}

function setCursor(input, position) {
  input.setSelectionRange(position, position);
}

/**
 *
 */

var AutoComplete = function (_React$Component) {
  inherits(AutoComplete, _React$Component);

  function AutoComplete() {
    var _temp, _this, _ret;

    classCallCheck(this, AutoComplete);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, _React$Component.call.apply(_React$Component, [this].concat(args))), _this), _this.state = {
      open: false,
      value: _this.props.defaultValue || '',
      input: null,
      currentSuggestions: [],
      selectedSuggestion: 0
    }, _this.handleChange = function (event) {
      var _this$props = _this.props;
      var children = _this$props.children;
      var limit = _this$props.limit;
      var _event$target = event.target;
      var value = _event$target.value;
      var selectionEnd = _event$target.selectionEnd;

      var completingValue = value.slice(0, selectionEnd);

      var completionTypes = React.Children.map(children, function (child) {
        return child.props;
      }).reduce(function (types, completionType) {
        var rx = getRegex(completionType);
        var match = rx.exec(completingValue);
        if (match) {
          var matchingValue = match[1] || match[0];
          if (matchingValue.length >= completionType.minLength) {
            types.push({
              type: completionType,
              regex: rx,
              matchingValue: matchingValue
            });
          }
        }
        return types;
      }, []);

      var currentSuggestions = completionTypes.reduce(function (available, childProps) {
        var type = childProps.type;
        var matchingValue = childProps.matchingValue;

        var currentLimit = limit - available.length;
        // Don't even ask for more completions if we've already reached our max.
        if (currentLimit <= 0) {
          return available;
        }
        var completions = type.getCompletions(matchingValue, type).slice(0, currentLimit);
        return [].concat(available, completions.map(function (completion) {
          return _extends({
            completion: completion
          }, childProps);
        }));
      }, []);

      _this.setState({
        value: value,
        input: event.target,
        currentSuggestions: currentSuggestions,
        selectedSuggestion: 0
      });

      var inputProps = _this.props.inputProps;

      if (inputProps.onChange && !event.defaultPrevented) {
        inputProps.onChange(event);
      }

      _this.sendUpdate(value);
    }, _this.handleKeyDown = function (event) {
      var _this$state = _this.state;
      var selectedSuggestion = _this$state.selectedSuggestion;
      var currentSuggestions = _this$state.currentSuggestions;

      if (event.key === 'ArrowUp') {
        event.preventDefault();
        if (currentSuggestions.length > 0) {
          var nextSuggestion = selectedSuggestion > 0 ? selectedSuggestion - 1 : currentSuggestions.length - 1;
          _this.setState({ selectedSuggestion: nextSuggestion });
        }
      } else if (event.key === 'ArrowDown') {
        event.preventDefault();
        if (currentSuggestions.length > 0) {
          var _nextSuggestion = (selectedSuggestion + 1) % currentSuggestions.length;
          _this.setState({ selectedSuggestion: _nextSuggestion });
        }
      } else if (event.key === 'Enter' || event.key === 'Tab') {
        if (currentSuggestions[selectedSuggestion]) {
          event.preventDefault();
          _this.select(selectedSuggestion, event.target);
        }
      } else if (event.key === 'ArrowLeft' || event.key === 'ArrowRight') {
        (function () {
          var target = event.target;

          setTimeout(function () {
            return _this.handleChange({ target: target });
          }, 1);
        })();
      }

      // Bubble up manually if necessary
      var inputProps = _this.props.inputProps;

      if (inputProps.onKeyDown && !event.defaultPrevented) {
        inputProps.onKeyDown(event);
      }
    }, _this.handleFocus = function (event) {
      _this.setState({ open: true });
      var inputProps = _this.props.inputProps;

      if (inputProps.onFocus) {
        inputProps.onFocus(event);
      }
    }, _this.handleBlur = function (event) {
      _this.setState({ open: false });
      var inputProps = _this.props.inputProps;

      if (inputProps.onBlur) {
        inputProps.onBlur(event);
      }
    }, _this.select = function (idx, input) {
      var _this$state2 = _this.state;
      var currentSuggestions = _this$state2.currentSuggestions;
      var value = _this$state2.value;

      var cursorPosition = input ? input.selectionEnd : value.length;
      var completion = currentSuggestions[idx];
      if (input) {
        (function () {
          var type = completion.type;
          var matchingValue = completion.matchingValue;

          var before = value.slice(0, cursorPosition - matchingValue.length);
          var after = value.slice(cursorPosition);
          var insertValue = type.getText(completion.completion, type);
          var newValue = '' + before + insertValue + after;

          _this.setState({
            value: newValue,
            currentSuggestions: []
          }, function () {
            input.focus();
            setCursor(input, before.length + insertValue.length);
          });

          _this.sendUpdate(newValue);
        })();
      }
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  AutoComplete.prototype.sendUpdate = function sendUpdate(value) {
    if (this.props.onUpdate) {
      this.props.onUpdate(value);
    }
  };

  AutoComplete.prototype.renderSuggestion = function renderSuggestion(_ref2, idx) {
    var _this2 = this;

    var type = _ref2.type;
    var completion = _ref2.completion;
    var _state = this.state;
    var input = _state.input;
    var selectedSuggestion = _state.selectedSuggestion;
    // Use the Completion type's suggestion renderer, or the default if it
    // doesn't have a custom one.

    var render = type.renderSuggestion || this.props.renderSuggestion;
    return render({
      key: '' + idx,
      value: completion,
      selected: idx === selectedSuggestion,
      select: function select() {
        return _this2.select(idx, input);
      }
    });
  };

  AutoComplete.prototype.renderSuggestions = function renderSuggestions() {
    var renderSuggestions = this.props.renderSuggestions;
    var currentSuggestions = this.state.currentSuggestions;


    var suggestions = currentSuggestions.map(this.renderSuggestion, this);
    return renderSuggestions(suggestions);
  };

  AutoComplete.prototype.render = function render() {
    var _state2 = this.state;
    var open = _state2.open;
    var currentSuggestions = _state2.currentSuggestions;
    var _props = this.props;
    var InputComponent = _props.inputComponent;
    var inputProps = _props.inputProps;


    var value = 'value' in this.props ? this.props.value : this.state.value;

    return React.createElement(
      'span',
      null,
      React.createElement(InputComponent, _extends({}, inputProps, {
        value: value,
        onChange: this.handleChange,
        onKeyDown: this.handleKeyDown,
        onFocus: this.handleFocus,
        onBlur: this.handleBlur
      })),
      open && currentSuggestions.length > 0 ? this.renderSuggestions() : null
    );
  };

  return AutoComplete;
}(React.Component);

AutoComplete.defaultProps = {
  inputComponent: 'input',
  inputProps: {
    type: 'text'
  },
  renderSuggestion: function renderSuggestion(_ref3) {
    var key = _ref3.key;
    var value = _ref3.value;
    var selected = _ref3.selected;
    var select = _ref3.select;
    return React.createElement(
      'div',
      {
        key: key,
        style: { fontWeight: selected ? 'bold' : 'normal' },
        onClick: select
      },
      value
    );
  },
  renderSuggestions: function renderSuggestions(suggestions) {
    return React.createElement(
      'div',
      null,
      suggestions
    );
  },
  children: [],
  limit: 15
};
process.env.NODE_ENV !== "production" ? AutoComplete.propTypes = {
  /**
   * Component to use for rendering the input element. Uses native `<input />`
   * by default.
   *
   * The component should accept `value`, `onChange` and `onKeyDown` props.
   */
  inputComponent: React.PropTypes.oneOfType([React.PropTypes.string, React.PropTypes.func]),
  /**
   * Props to pass to the input component.
   */
  inputProps: React.PropTypes.object,
  /**
   * Function that renders a single suggestion. This can be overridden for
   * individual Completion types, in case they need custom rendering.
   *
   * @default `<div key={key} onClick={select}>{value}</div>`
   *
   * @param {Object} suggestion - Suggestion descriptor.
   * @param {string} suggestion.key - Unique key for the suggestion element.
   *     See [Dynamic Children](https://facebook.github.io/react/docs/multiple-components.html#dynamic-children)
   *     for details.
   * @param {*} suggestion.value - Completion value of this suggestion.
   * @param {boolean} suggestion.selected - Whether this suggestion is
   *     currently selected.
   * @param {function} suggestion.select - Autocomplete this suggestion.
   * @returns {element}
   */
  renderSuggestion: React.PropTypes.func,
  /**
   * Function that renders the suggestions list.
   *
   * @default `<div>{suggestions}</div>`
   *
   * @param {Array} suggestions - Array of children rendered by
   *     `renderSuggestion`.
   * @returns {element}
   */
  renderSuggestions: React.PropTypes.func,
  /**
   * Completion types as [`<Completion />`][Completion] elements.
   */
  children: React.PropTypes.node,
  /**
   * The maximum amount of suggestions to show.
   */
  limit: React.PropTypes.number,
  /**
   * Current string value of the input component. Optional, useful for
   * controlled inputs. Passed down to the input component as the value prop.
   */
  value: React.PropTypes.string,
  /**
   * Initial string value for uncontrolled inputs.
   */
  defaultValue: React.PropTypes.string,
  /**
   * Fired when the input component's value changes. Use this for controlled
   * inputs.
   *
   * @param {string} newValue
   */
  onUpdate: React.PropTypes.func
} : void 0;

exports.Completion = Completion;
exports['default'] = AutoComplete;